@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """This module provides the core primitives of Hypothesis, such as given."""
 
@@ -105,6 +100,7 @@ from hypothesis.reporting import (
 )
 from hypothesis.statistics import describe_targets, note_statistics
 from hypothesis.strategies._internal.collections import TupleStrategy
+from hypothesis.strategies._internal.misc import NOTHING
 from hypothesis.strategies._internal.strategies import (
     Ex,
     MappedSearchStrategy,
@@ -246,9 +242,9 @@ def is_invalid_test(test, original_argspec, given_arguments, given_kwargs):
     only be reported for tests that actually ran.
     """
 
-    def invalid(message):
+    def invalid(message, *, exc=InvalidArgument):
         def wrapped_test(*arguments, **kwargs):
-            raise InvalidArgument(message)
+            raise exc(message)
 
         wrapped_test.is_hypothesis_test = True
         wrapped_test.hypothesis = HypothesisHandle(
@@ -306,6 +302,18 @@ def is_invalid_test(test, original_argspec, given_arguments, given_kwargs):
             )
         )
 
+    # This case would raise Unsatisfiable *anyway*, but by detecting it here we can
+    # provide a much more helpful error message for people e.g. using the Ghostwriter.
+    empty = [
+        f"{s!r} (arg {idx})" for idx, s in enumerate(given_arguments) if s is NOTHING
+    ] + [f"{name}={s!r}" for name, s in given_kwargs.items() if s is NOTHING]
+    if empty:
+        strats = "strategies" if len(empty) > 1 else "strategy"
+        return invalid(
+            f"Cannot generate examples from empty {strats}: " + ", ".join(empty),
+            exc=Unsatisfiable,
+        )
+
 
 class ArtificialDataForExample(ConjectureData):
     """Dummy object that pretends to be a ConjectureData object for the purposes of
@@ -941,8 +949,6 @@ class HypothesisHandle:
 
         Returns None if buffer invalid for the strategy, canonical pruned
         bytes if the buffer was valid, and leaves raised exceptions alone.
-
-        Note: this feature is experimental and may change or be removed.
         """
         # Note: most users, if they care about fuzzer performance, will access the
         # property and assign it to a local variable to move the attribute lookup
