@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import math
 from typing import Any, Mapping, Optional, Sequence, Tuple, Union
@@ -80,6 +75,7 @@ def from_dtype(
     max_value: Union[int, float, None] = None,
     allow_nan: Optional[bool] = None,
     allow_infinity: Optional[bool] = None,
+    allow_subnormal: Optional[bool] = None,
     exclude_min: Optional[bool] = None,
     exclude_max: Optional[bool] = None,
 ) -> st.SearchStrategy[Any]:
@@ -131,6 +127,7 @@ def from_dtype(
                 "max_value",
                 "allow_nan",
                 "allow_infinity",
+                "allow_subnormal",
                 "exclude_min",
                 "exclude_max",
             ),
@@ -171,7 +168,7 @@ def from_dtype(
             # it here because we'd have to guard against equivalents in arrays()
             # regardless and drawing scalars is a valid use-case.
             res = st.sampled_from(TIME_RESOLUTIONS)
-        result = st.builds(dtype.type, st.integers(-(2 ** 63), 2 ** 63 - 1), res)
+        result = st.builds(dtype.type, st.integers(-(2**63), 2**63 - 1), res)
     else:
         raise InvalidArgument(f"No strategy inference for {dtype}")
     return result.map(dtype.type)
@@ -197,6 +194,21 @@ class ArrayStrategy(st.SearchStrategy):
             ) from err
         if self._check_elements and val != result[idx] and val == val:
             strategy = self.fill if fill else self.element_strategy
+            if self.dtype.kind == "f":
+                try:
+                    is_subnormal = 0 < abs(val) < np.finfo(self.dtype).tiny
+                except Exception:  # pragma: no cover
+                    # val may be a non-float that does not support the
+                    # operations __lt__ and __abs__
+                    is_subnormal = False  # pragma: no cover
+                if is_subnormal:
+                    raise InvalidArgument(  # pragma: no cover
+                        f"Generated subnormal float {val} from strategy "
+                        f"{strategy} resulted in {result[idx]!r}, probably "
+                        "as a result of NumPy being built with flush-to-zero "
+                        "compiler options. Consider passing "
+                        "allow_subnormal=False."
+                    )
             raise InvalidArgument(
                 "Generated array element %r from %r cannot be represented as "
                 "dtype %r - instead it becomes %r (type %r).  Consider using a more "
@@ -255,7 +267,10 @@ class ArrayStrategy(st.SearchStrategy):
                 # sqrt isn't chosen for any particularly principled reason. It
                 # just grows reasonably quickly but sublinearly, and for small
                 # arrays it represents a decent fraction of the array size.
-                average_size=math.sqrt(self.array_size),
+                average_size=min(
+                    0.9 * self.array_size,  # ensure small arrays sometimes use fill
+                    max(10, math.sqrt(self.array_size)),  # ...but *only* sometimes
+                ),
             )
 
             needs_fill = np.full(self.array_size, True)
@@ -312,7 +327,7 @@ class ArrayStrategy(st.SearchStrategy):
             if mismatch.any():
                 raise InvalidArgument(
                     "Array elements %r cannot be represented as dtype %r - instead "
-                    "they becomes %r.  Use a more precise strategy, e.g. without "
+                    "they become %r.  Use a more precise strategy, e.g. without "
                     "trailing null bytes, as this will be an error future versions."
                     % (result[mismatch], self.dtype, out[mismatch])
                 )
@@ -860,6 +875,7 @@ def basic_indices(
         max_dims=max_dims,
         allow_ellipsis=allow_ellipsis,
         allow_newaxis=allow_newaxis,
+        allow_fewer_indices_than_dims=True,
     )
 
 
