@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import codecs
 import inspect
@@ -19,6 +14,14 @@ import platform
 import sys
 import typing
 
+try:
+    BaseExceptionGroup = BaseExceptionGroup
+except NameError:  # pragma: no cover
+    try:
+        from exceptiongroup import BaseExceptionGroup as BaseExceptionGroup  # for mypy
+    except ImportError:
+        BaseExceptionGroup = ()  # valid in isinstance and except clauses!
+
 PYPY = platform.python_implementation() == "PyPy"
 WINDOWS = platform.system() == "Windows"
 
@@ -39,20 +42,6 @@ def int_to_byte(i):
     return bytes([i])
 
 
-try:
-    # These types are new in Python 3.7, but also (partially) backported to the
-    # typing backport on PyPI.  Use if possible; or fall back to older names.
-    typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
-    ForwardRef = typing.ForwardRef  # type: ignore
-except AttributeError:  # pragma: no cover
-    typing_root_type = (typing.TypingMeta, typing.TypeVar)  # type: ignore
-    try:
-        typing_root_type += (typing._Union,)  # type: ignore
-    except AttributeError:
-        pass
-    ForwardRef = typing._ForwardRef  # type: ignore
-
-
 def is_typed_named_tuple(cls):
     """Return True if cls is probably a subtype of `typing.NamedTuple`.
 
@@ -88,13 +77,11 @@ def get_type_hints(thing):
     except (AttributeError, TypeError, NameError):
         hints = {}
 
-    if not inspect.isclass(thing):
-        return hints
-
-    try:
-        hints.update(typing.get_type_hints(thing.__init__, **kwargs))
-    except (TypeError, NameError, AttributeError):
-        pass
+    if inspect.isclass(thing):
+        try:
+            hints.update(typing.get_type_hints(thing.__init__, **kwargs))
+        except (TypeError, NameError, AttributeError):
+            pass
 
     try:
         if hasattr(thing, "__signature__"):
